/*
**  NoteWindowController.m
**
**  Copyright (c) 2001
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "NoteWindowController.h"

#import "Affiche.h"
#import "Constants.h"
#import "Note.h"
#import "NoteInfoWindowController.h"

#ifndef MACOSX
#import "NoteWindow.h"
#endif

static id lastNoteWindowOnTop = nil;

@implementation NoteWindowController

- (id) initWithWindowNibName: (NSString *) theNibName
{
#ifdef MACOSX
  self = [super initWithWindowNibName: theNibName];
#else
  NoteWindow *noteWindow;
  
  noteWindow = [[NoteWindow alloc] initWithContentRect:NSMakeRect(10,10,200,100)
                                   styleMask: (NSClosableWindowMask|NSResizableWindowMask)
                                   backing: NSBackingStoreBuffered
                                   defer: NO];

  self = [super initWithWindow: noteWindow];
  
  [noteWindow layoutWindow];
  [noteWindow setDelegate: self];

  // We link our outlets
  textView = [noteWindow textView];

#endif
  
  // We set the title to an empty string
  [[self window] setTitle: @""];
  
  return self;
}

- (void) dealloc
{
  lastNoteWindowOnTop = nil;
  
  RELEASE(note);
  
  [super dealloc];
}

//
// delegate methods
//

- (void) textDidChange: (NSNotification *) aNotification
{
  NSAttributedString *anAttributedString;
  
  anAttributedString = [[NSAttributedString alloc] initWithAttributedString: [textView textStorage]];
  [[self note] setValue: anAttributedString];
  
  [self _updateWindowTitle];

  RELEASE(anAttributedString);
}


- (void) windowDidBecomeMain: (NSNotification *) aNotification
{
  lastNoteWindowOnTop = [self window];
  [[NoteInfoWindowController singleInstance] setNote: [self note]];
}


- (void) windowDidLoad
{
  lastNoteWindowOnTop = [self window];
  
  [super windowDidLoad];
}


- (void) windowDidMove: (NSNotification *) aNotification
{
  [self _updateWindowFrame];
}


- (void) windowDidResize: (NSNotification *) aNotification
{
  [self _updateWindowFrame];
}

- (BOOL) windowShouldClose: (id) sender
{
  int choice;

  choice = NSRunAlertPanel(_(@"Closing a note..."),
			   _(@"Would you like to save this note to a file?"),
			   _(@"Cancel"), // default
			   _(@"Yes"),    // alternate
			   _(@"No"));    // other return 
  
  if (choice == NSAlertDefaultReturn)
    {
      return NO;
    }
  else if ( choice == NSAlertAlternateReturn )
    {
      NSSavePanel *aSavePanel;
      int aChoice;
      
      aSavePanel = [NSSavePanel savePanel];
      [aSavePanel setAccessoryView: nil];
      [aSavePanel setRequiredFileType: @""];
      
      aChoice = [aSavePanel runModalForDirectory:NSHomeDirectory() file: @"note.txt"];
      
      /* if successful, save file under designated name */
      if (aChoice == NSOKButton)
	{
	  if (! [[textView string] writeToFile: [aSavePanel filename]
				   atomically: YES] )
	    {
	      NSBeep();
	    }
	}
    }
  
  [(Affiche *)[NSApp delegate] deleteNote: [self note]];
  [(Affiche *)[NSApp delegate] synchronize];

  return YES;
}

- (void) windowWillClose: (NSNotification *) theNotification
{
  AUTORELEASE(self);
}


//
// action methods
//


//
// access / mutation
//

+ (id) lastNoteWindowOnTop
{
  return lastNoteWindowOnTop;
}

- (Note *) note
{
  return note;
}

- (void) setNote: (Note *) theNote
{
  if ( theNote )
    {
      RETAIN(theNote);
      RELEASE(note);
      note = theNote;
      
      [[textView textStorage] setAttributedString: [note value]];
      [textView setBackgroundColor: [note backgroundColor]];
      [[self window] setFrame: [note frame]
		     display: NO];
      
      [textView setNeedsDisplay: YES];
      [self _updateWindowTitle];
    }
  else
    {
      RELEASE(note);
      note = nil;
    }
}

- (NSTextView *) textView
{
  return textView;
}

@end


//
// private methods
//
@implementation NoteWindowController (Private)

- (void) _updateWindowFrame
{
  if ( [self note] )
    {
      [[self note] setFrame: [[self window] frame]];
    }
}


- (void) _updateWindowTitle
{
  NSArray *allLines;

  allLines = [[textView string] componentsSeparatedByString: @"\n"];

  if ( [allLines count] > 0 )
    {
      [[self window] setTitle: [allLines objectAtIndex: 0]];
    }
}

@end
